close all;

example = 4;

if (example == 1)
    % g = 10 + 2*z1 + 3*z2;
    % g ~N(10,3.6056);
    % Failure g < 0
    
    Pf_Exact = Normal_Probability(10,3.6056,0);
    
    % Using failure space ideas
    z1 = randn(1,1e6);
    z2 = randn(1,1e6);
    
    g = 10 + 2*z1 + 3*z2;
    
    % Failure g < 0
    
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    
    dist = sqrt(z1(indices).^2 + z2(indices).^2);
    [beta,index] = min(dist);
    closestPt = [z1(indices(index)) z2(indices(index))];
    plot([0 closestPt(1)],[0 closestPt(2)]);
    
    Pf_FailureSpace = 1 - Normal_Probability(0,1,beta);
elseif (example == 2)
    %g = x2 - x1./(1+0.01*x1);
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    mu1 = 23; sigma1 = 2;
    mu2 = 32; sigma2 = 4;
    x1 = mu1 + sigma1*z1; % same as x1 = Normal_Sampling(23,2,1e6);
    x2 = mu2 + sigma2*z2;% same as x2 = Normal_Sampling(32,4,1e6);
    g = x2 - x1./(1+0.01*x1);
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    
    dist = sqrt(z1(indices).^2 + z2(indices).^2);
    [beta,index] = min(dist);

    closestPt = [z1(indices(index)) z2(indices(index))];
    plot([0 closestPt(1)],[0 closestPt(2)]);
    plot(closestPt(1),closestPt(2),'bd');
    
    Pf_FailureSpace = 1 - Normal_Probability(0,1,beta);
elseif (example == 3) 
    %g = x2 - x1./(1+0.01*x1);
    zBar = [0 0]; % initial guess for closest point to failure space
  
    plot(zBar(1),zBar(2),'k*');
    axis([-1 4 -4 1]); axis equal;hold on;
    
    grid on;
    for i = 1:5
        [g,grad_g] = gNonlinearEx1(zBar);
        
        a1 = grad_g(1);
        a2 = grad_g(2);
        a0 = g - a1*zBar(1) - a2*zBar(2);
        
        
        zBar(1) = -a1*a0/(a1*a1+a2*a2);
        zBar(2) = -a2*a0/(a1*a1+a2*a2);
        
        
        plot(zBar(1),zBar(2),'r*');
        
        beta = a0/sqrt(a1*a1+a2*a2);
        Pf_HL = 1- Normal_Probability(0,1,beta);
    end
elseif (example == 4)
    % g = 1.1 + a*z1.^2 - z2;
    nSamples = 1e5;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    a = 0.2;
    g = 1.1 + a*z1.^2 - z2;
    
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis equal;hold on;
    plot(0,0,'k*');
    grid on;

    dist = sqrt(z1(indices).^2 + z2(indices).^2);
    [beta,index] = min(dist);
    closestPt = [z1(indices(index)) z2(indices(index))];
    plot([0 closestPt(1)],[0 closestPt(2)]);
    plot(closestPt(1),closestPt(2),'bd');
    
    Pf_HL = 1 - Normal_Probability(0,1,beta);
    
elseif (example == 5)
    %g = 1.1 + 0.2*z1.^2 +0.25*z2.^2 - z3;
    nSamples = 2e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    z3 = randn(1,nSamples);
    
    g = 1.1 + 0.2*z1.^2 +0.25*z2.^2 - z3;
    
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    indices = find(g<0);
    plot3(z1(indices),z2(indices),z3(indices),'r.');
    axis equal;hold on;
    plot(0,0,'k*');
    grid on;

    dist = sqrt(z1(indices).^2 + z2(indices).^2+ z3(indices).^2);
    [~,index] = min(dist);
    closestPt = [z1(indices(index)) z2(indices(index)) z3(indices(index))];
    plot3([0 closestPt(1)],[0 closestPt(2)],[0 closestPt(3)]);
    plot(closestPt(1),closestPt(2),'bd');
    
    % H-L algorithm
    [beta, zStar, pf,nFunctionCalls,flag] = HansoferLindND(@(zVec)gNonlinearEx3(zVec),3);

    Pf_HL = 1 - Normal_Probability(0,1,beta);

    % Brietung correction
    grad_g = [0.4*zStar(1) 0.5*zStar(2) -1];
    
    H = [0.4 0 0; 0 0.5 0; 0 0 0];
    
    A = zeros(4,4);
    A(1:3,1:3) = H;
    A(4,1:3) = grad_g(:)';
    A(1:3,4) = grad_g(:);
    
    B = zeros(4,4);
    B(1:3,1:3) = eye(3,3);

    Lambda = eig(A,B);

    kappa1 = 0.4/norm(grad_g);
    kappa2 = 0.5/norm(grad_g);

    Pf_Brietung = Pf_HL/sqrt((1+beta*kappa1)*(1+beta*kappa2));

end

