
close all;
example = 2;
if (example == 1) % g1
    disp('g1');
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Naive MC estimation
    g = gNonlinearEx4([z1;z2]);
  
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    % Failure space plotting
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    disp(['Pf_MC = ' num2str(Pf_MC)]);
elseif (example == 2)
    disp('g2');
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Naive MC estimation
    g = gNonlinearEx5([z1;z2]);
  
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    % Failure space plotting
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    disp(['Pf_MC = ' num2str(Pf_MC)]);
elseif (example == 3)
    disp('g1 or g2');
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Naive MC estimation
    g1 = gNonlinearEx4([z1;z2]);
    g2 = gNonlinearEx5([z1;z2]);
    g = g1 + g2 -sqrt(g1.^2 + g2.^2); % or g = min(g1,g2);
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    % Failure space plotting
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    disp(['Pf_MC = ' num2str(Pf_MC)]);
elseif (example == 4)
    disp('g1 and g2');
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    g1 = gNonlinearEx4([z1;z2]);
    g2 = gNonlinearEx5([z1;z2]);
    g = (g1 + g2 + sqrt(g1.^2 + g2.^2))/2;% g = max(g1,g2); g = (g1 + g2 + sqrt(g1.^2 + g2.^2))/2;
  
    Pf_MC = sum(g<0)/numel(g); % probability of failure
    
    % Failure space plotting
    indices = find(g<0);
    plot(z1(indices),z2(indices),'r.');
    axis([-1 4 -4 1]); axis equal;hold on;
    plot(0,0,'k*');
    grid on;
    disp(['Pf_MC = ' num2str(Pf_MC)]);
elseif (example == 5)
    [beta, zStar, Pf_HL,nFunctionEvals,flag]= ...
        HansoferLind2D(@(zVec)gOrFunction(zVec));
elseif (example == 6) % Use SE for composition
    disp('g4 or g5');
    
    % use HL algorithm to find out beta for g4
    [beta1, zStar1, Pf_HL1,nFunctionEvals1,flag1]=HansoferLind2D(@(zVec)gNonlinearEx4(zVec));
    
     % use HL algorithm to find out beta for g5
    [beta2, zStar2, Pf_HL2,nFunctionEvals2,flag2]=HansoferLind2D(@(zVec)gNonlinearEx5(zVec));
    
    disp(['beta1 = ' num2str(beta1)]);
    disp(['beta2 = ' num2str(beta2)]);

    % Use the smaller of the two 
    beta = min(beta1, beta2);
    
    % create samples as before.
    
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Only consider samples whose distance is greater than or equal to beta
    d = sqrt(z1.^2 + z2.^2);
    indices  = find(d >= beta);
    
    % Evaluate the two g functions outside the circle
    g1 = gNonlinearEx4([z1(indices);z2(indices)]);
    g2 = gNonlinearEx5([z1(indices);z2(indices)]);
    
    % Composition
    g = g1 + g2 -sqrt(g1.^2 + g2.^2); % g = min(g1,g2); g = g1 + g2 -sqrt(g1.^2 + g2.^2);
  

    Pf_SE = sum(g<0)/nSamples; % probability of failure
    disp(['Pf_SE = ' num2str(Pf_SE)]);
    disp(['nFunctionEvals = ' num2str(2*length(indices))]);
elseif (example == 7) % Use SE for composition
    disp('g4 and g5');
    
    % use HL algorithm to find out beta for g4
    [beta1, zStar1, Pf_HL1,nFunctionEvals1,flag1]=HansoferLind2D(@(zVec)gNonlinearEx4(zVec));
    
     % use HL algorithm to find out beta for g5
    [beta2, zStar2, Pf_HL2,nFunctionEvals2,flag2]=HansoferLind2D(@(zVec)gNonlinearEx5(zVec));
    
    disp(['beta1 = ' num2str(beta1)]);
    disp(['beta2 = ' num2str(beta2)]);

    % Use the larger of the two 
    beta = max(beta1, beta2);
    
    % create samples as before.
    
    nSamples = 1e6;
    z1 = randn(1,nSamples);
    z2 = randn(1,nSamples);
    
    % Only consider samples whose distance is greater than or equal to beta
    d = sqrt(z1.^2 + z2.^2);
    indices  = find(d >= beta);
    
    % Evaluate the two g functions outside the circle
    g1 = gNonlinearEx4([z1(indices);z2(indices)]);
    g2 = gNonlinearEx5([z1(indices);z2(indices)]);
    
    % Composition
    g = g1 + g2 + sqrt(g1.^2 + g2.^2); % g = min(g1,g2); g = g1 + g2 -sqrt(g1.^2 + g2.^2);
  

    Pf_SE = sum(g<0)/nSamples; % probability of failure
    disp(['Pf_SE = ' num2str(Pf_SE)]);
    disp(['nFunctionEvals = ' num2str(2*length(indices))]);
end
