classdef reliabilityTestFunctions
    % This class encapsulates numerous test functions
    methods(Static)
        function [f,g,h] = f1(x)
            % returns  (x-1)^2 and its derivatives
            f = (x-1).^2;
            if (nargout > 1), g = 2*(x-1);end
            if (nargout > 2), h = 2;end
        end
        function [f,g,h] = f2(x)
            % returns  (x-1)^2 and its derivatives
            f = (x-1).^3;
            if (nargout > 1), g = 3*(x-1).^2;end
            if (nargout > 2), h = 6*(x-1);end
        end
        function [g,gradZ,hessZ] = g1(z)
            x1 = z(1,:) + 10;
            x2 = z(2,:) + 13;
            g = x2-x1;
            if (nargout > 1)
                gradZ = [-1 1]';
            end
            if (nargout > 2)
                hessZ = [0 0; 0 0];
            end
        end
        function [gValue,gradZ,hessZ] = g2(z)
            x1 = z(1,:)+10;
            x2 = z(2,:)+13;
            gValue = x2  - x1./(1+0.01*x1);
            if (nargout > 1)
                gradZ = [-10000/(x1 + 100).^2; 1];
            end
            if (nargout > 2)
                hessZ = [0 0; 0 0]; % not implemented
            end
        end
        
        function [g,gradZ,hessZ]  = g3(z)
            x1 = 3*z(1,:)+10;
            x2 = 3*z(2,:)+10;
            g = 2.2257 - 0.001309*(x1+x2-20).^3 + 0.23571*(x1-x2);
            if (nargout > 1)
                grad_x(1,1) = 0.23571 - (0.003927*(x1 + x2 - 20)^2);
                grad_x(2,1) = - (0.003927*(x1 + x2 - 20)^2) - 0.23571;
                gradZ = 3*grad_x;
            end
            if (nargout > 2)
                hessZ = [0 0; 0 0]; % not implemented
            end
            
        end
        function [gValue,gradZ,hessZ]  = g4(z)
            x1 = 3*z(1,:)+20;
            x2 = 4*z(2,:)+26;
            gValue = x2  - x1./(1+0.01*x1);
            if (nargout > 1)
                gradg_x = [-10000/(x1 + 100).^2; 1];
                gradZ = [3 0; 0 4]*gradg_x;
            end
            if (nargout > 2)
                hessZ = [0 0; 0 0]; % not implemented
            end
        end
        function [gValue,gradZ,hessZ]  = g5(z)
            x1 = 3*z(1,:)+20;
            x2 = 4*z(2,:)+26;
            x3 = 2*z(3,:)+50;
            gValue = x3  - x1./(1+0.01*x1) - x2./(1+0.01*x2);
            if (nargout > 1)
                gradg_x = [-10000/(x1 + 100).^2; -10000/(x2 + 100).^2; 1];
                gradZ = [3 0 0; 0 4 0; 0 0 2]*gradg_x;
            end
            if (nargout > 2)
                hessZ = zeros(3,3);
            end
        end
        function [gValue,grad]  = g6(z)
            x1 = 3*z(1,:)+20;
            x2 = 4*z(2,:)+26;
            x3 = 2*z(3,:)+50;
            gValue = x3  - x1./(1+0.01*x1) - x2./(1+0.01*x2);
            if (nargout > 1)
                grad = complexVariableGradient(@g6,z);
            end
        end
        function [gValue,gradgZ,hessZ] = g7(z)
            a = 0.05;
            gValue = 3+z(2)/(1+a*z(1))  - z(1)./(1+a*z(2)^2);
            if (nargout > 1)
                gradgZ = complexVariableGradient(@(z)g7(z),z);
            end
            if (nargout > 2)
                [~,~,hessZ] = complexDerivatives(@(z)g7(z),z);
            end
        end
        function [gValue,gradZ,hessZ] = g8a(z)
            a = 0.2;
            gValue = 1.1 + a*z(1,:).^2 - z(2,:);
            if (nargout > 1)
                gradZ = [2*a*z(1); - 1];
            end
            if (nargout > 2)
                hessZ = [2*a 0; 0 0];
            end
        end
        function [gValue,gradZ,hessZ] = g8b(z)
            a = 0.6;
            gValue = 2.6 + a*z(1,:).^2 - z(2,:);
            if (nargout > 1)
                gradZ = [2*a*z(1); - 1];
            end
            if (nargout > 2)
                hessZ = [2*a 0; 0 0];
            end
        end
        function [gValue,gradZ,hessZ] = g9(z)
            a = 0.05;
            b = 0.05;
            gValue = 1.1 + a*z(1,:).^2 + b*z(2,:).^2 - z(3,:);
            if (nargout > 1)
                gradZ = [2*a*z(1); 2*b*z(1);- 1];
            end
            if (nargout > 2)
                hessZ = [2*a 0 0; 0 2*b 0; 0 0 0];
            end
        end
        function [gValue] = g10(z)
            a = 0.2;
            g1= 1.1 + a*z(1,:).^2 - z(2,:);
            g2 = 1.5 - z(1,:);
            gValue = g1 + g2 - sqrt(g1.^2 + g2.^2); % R function boolean
        end
        function [g,gradZ,hessZ] = gBeamStress(z)
            sMax = 400;
            P = 100;
            L = 2000;%mm
            mu1 = 10;
            mu2 = 20;
            sigma1 = 0.5;
            sigma2 = 1.5;
            g = sMax - 6*P*L/(sigma1*z(1)+mu1)/(sigma2*z(2)+mu2)^2;
            if (nargout > 1)
                gradZ(1,1) = 6*sigma1*P*L/(sigma1*z(1)+mu1)^2/(sigma2*z(2)+mu2)^2;
                gradZ(2,1) = 6*2*sigma2*P*L/(sigma1*z(1)+mu1)/(sigma2*z(2)+mu2)^3;
            end
            if (nargout > 2)
                g11 = -2*6*sigma1^2*P*L/(sigma1*z(1)+mu1)^3/(sigma2*z(2)+mu2)^2;
                g12 = -2*6*sigma1*sigma2*P*L/(sigma1*z(1)+mu1)^2/(sigma2*z(2)+mu2)^3;
                g21 = g12;
                g22 = -3*6*2*sigma2^2*P*L/(sigma1*z(1)+mu1)/(sigma2*z(2)+mu2)^4;
                hessZ = [g11 g12; g21 g22];
            end
            
        end
        function [g] = gComposition1(z)
            % See R-Functions, A Primer by Vadim Shapiro
            % https://ecommons.cornell.edu/bitstream/handle/1813/7059/91-1219.pdf?sequence=1&isAllowed=y
            g1 = (-z(1) -1);
            g2 = (z(1)-1);
            g3 = 1-z(2);
            g = BooleanAnd(BooleanAnd(g1,g2),g3);
        end
        function [g] = gComposition2(z)
            % See R-Functions, A Primer by Vadim Shapiro
            % https://ecommons.cornell.edu/bitstream/handle/1813/7059/91-1219.pdf?sequence=1&isAllowed=y
            g1 = (-z(1) -1);
            g2 = (z(1)-1);
            g3 = 1 + z(1) -z(2);
            g4 = 1 - z(1) -z(2);
            g = BooleanAnd(BooleanAnd(BooleanOr(g3,g4),g1),g2);
        end
    end
end
