% Monte Carlo analysis of Hoeken's linkage
angles = 94:2:266;
deviation = zeros(1,numel(angles));
idealY = 4.0;
allowedDeviation  = 0.05;

L0 =[1 2.5 2.5 2 2.5]; % nominal 
N = 20000; % number of samples
% All lengths are normally distributed (and statistically independent)
% with mean as ideal value and coefficient of variation
COV = 0.005;
sigmaL1 = COV*L0(1);
sigmaL2 = COV*L0(2);
sigmaL3 = COV*L0(3);
sigmaL4 = COV*L0(4);
sigmaL5 = COV*L0(5);

% Generate samples
L1Samples = N_Sampling(L0(1),sigmaL1,N);
L2Samples = N_Sampling(L0(2),sigmaL2,N);
L3Samples = N_Sampling(L0(3),sigmaL3,N);
L4Samples = N_Sampling(L0(4),sigmaL4,N);
L5Samples = N_Sampling(L0(5),sigmaL5,N);

nf = 0; % Keep track of number of failures
for n = 1:N
    L(1) = L1Samples(n);
    L(2) = L2Samples(n);
    L(3) = L3Samples(n);
    L(4) = L4Samples(n);
    L(5) = L5Samples(n);
    for i = 1:numel(angles)
        P = HoekensLinkage(angles(i)*pi/180,L);
        deviation(i) = (P(2,5)-idealY); % find the deviation from ideal
    end
    maxDeviation = max(abs(deviation)); % note absolute
    if (maxDeviation > allowedDeviation)
        nf = nf + 1; % increment failure count
    end
end
pf = nf/N
